package com.supervision.minio.client;

import cn.hutool.core.io.IoUtil;
import com.supervision.minio.exception.SysObsException;
import io.minio.*;
import io.minio.errors.MinioException;
import io.minio.http.Method;
import io.minio.messages.Bucket;
import io.minio.messages.DeleteError;
import io.minio.messages.DeleteObject;
import io.minio.messages.Item;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * minio 操作Template
 *
 * @author qmy
 * @version 1.0.0  2021/6/22 14:42
 * @since JDK1.8
 */
@Component
@Slf4j
public class MinioTemplate {

    @Autowired
    private MinioClient minioClient;

    /**
     * 上传对象
     *
     * @param bucketName   存储桶名称
     * @param objectName   对象名称
     * @param userMetadata metaData
     * @param stream       流
     * @param objectSize   大小
     * @param contentType  contentType
     * @return void
     * @author qmy
     * @version 1.0.0  2021/6/30 9:11
     * @since JDK1.8
     */
    public void putObject(String bucketName, String objectName, Map<String, String> userMetadata, InputStream stream, long objectSize, String contentType) {
        try {
            minioClient.putObject(PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .userMetadata(userMetadata)
                    .contentType(contentType == null ? "application/octet-stream" : contentType)
                    .stream(stream, objectSize, -1)
                    .build());
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 下载对象
     *
     * @param bucketName 存储桶名称
     * @param objectName 对象名称
     * @param response   response
     * @return void
     * @author qmy
     * @version 1.0.0  2021/6/30 9:11
     * @since JDK1.8
     */
    public void getObject(String bucketName, String objectName, String fileType, HttpServletResponse response) {
        try {
            InputStream inputStream = minioClient.getObject(
                    GetObjectArgs.builder().bucket(bucketName).object(objectName).build());

            StatObjectResponse statObjectResponse = minioClient.statObject(
                    StatObjectArgs.builder().bucket(bucketName).object(objectName).build());

            String fileName = objectName + "." + fileType;
            response.setHeader("Content-Disposition", "inline;filename=" + fileName);
            response.setCharacterEncoding("UTF-8");
            response.setContentLengthLong(statObjectResponse.size());
            response.setContentType(statObjectResponse.contentType());

            IoUtil.copy(inputStream, response.getOutputStream());
            inputStream.close();
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 下载对象
     *
     * @param bucketName 存储桶名称
     * @param objectName 对象名称
     * @return void
     * @author qmy
     * @version 1.0.0  2021/6/30 9:11
     * @since JDK1.8
     */
    public GetObjectResponse getInputStreamObject(String bucketName, String objectName) {
        GetObjectResponse inputStream;
        try {
             inputStream = minioClient.getObject(
                    GetObjectArgs.builder().bucket(bucketName).object(objectName).build());
            return inputStream;
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 删除对象
     *
     * @param bucketName 存储桶名称
     * @param objectName 对象名称
     * @return void
     * @author qmy
     * @version 1.0.0  2021/6/30 9:08
     * @since JDK1.8
     */
    public void removeObject(String bucketName, String objectName) {
        try {
            minioClient.removeObject(
                    RemoveObjectArgs.builder().bucket(bucketName).object(objectName).build());
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 删除对象
     *
     * @param bucketName  存储桶名称
     * @param objectNames 对象名称
     * @return void
     * @author qmy
     * @version 1.0.0  2021/6/30 9:08
     * @since JDK1.8
     */
    public List<Map<String, Object>> removeObject(String bucketName, Set<String> objectNames) {
        try {
            List<DeleteObject> objects = new LinkedList<>();
            for (String objectName : objectNames) {
                objects.add(new DeleteObject(objectName));
            }
            Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                    RemoveObjectsArgs.builder().bucket(bucketName).objects(objects).build());
            List<Map<String, Object>> errorInfo = new ArrayList<>();
            for (Result<DeleteError> result : results) {
                DeleteError error = result.get();
                errorInfo.add(new HashMap<String, Object>(2) {{
                    put("objectName", error.objectName());
                    put("message", error.message());
                }});
            }
            return errorInfo;
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 获取对象信息
     *
     * @param bucketName 存储桶名称
     * @param objectName 对象名称
     * @return io.minio.StatObjectResponse
     * @author qmy
     * @version 1.0.0  2021/6/30 9:07
     * @since JDK1.8
     */
    public StatObjectResponse statObject(String bucketName, String objectName) {
        try {
            return minioClient.statObject(
                    StatObjectArgs.builder().bucket(bucketName).object(objectName).build());
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new SysObsException(e.getMessage());
        }
    }

    /**
     * 对象是否存在
     *
     * @param bucketName 存储桶名称
     * @param objectName 对象名称
     * @return boolean
     * @author qmy
     * @version 1.0.0  2021/6/30 9:15
     * @since JDK1.8
     */
    public boolean objectExists(String bucketName, String objectName) {
        try {
            return minioClient.statObject(
                    StatObjectArgs.builder().bucket(bucketName).object(objectName).build()) != null;
        } catch (MinioException | InvalidKeyException | IOException | NoSuchAlgorithmException e) {
            return false;
        }
    }

}
